@@ -2,7 +2,8 @@
 A framework agnostic implementation for storage of MCMC draws.
 """
 from .backends.numpy import NumPyBackend
-from .core import Backend, Chain, ChainMeta, Run, RunMeta
+from .core import Backend, Chain, Run
+from .meta import ChainMeta, Coordinate, DataVariable, ExtendedValue, RunMeta, Variable
 
 # Backends
 try:
@@ -19,4 +20,4 @@ except ModuleNotFoundError:
     pass
 
 
-__version__ = "0.1.0"
+__version__ = "0.1.2"
